@@ -104,6 +104,7 @@ ui_initialize_options(struct mknoiseparams *p,
 
 
   /* Set the necessary common parameters structure. */
+  cp->program_struct     = p;
   cp->program_name       = PROGRAM_NAME;
   cp->program_exec       = PROGRAM_EXEC;
   cp->program_bibtex     = PROGRAM_BIBTEX;
@@ -284,8 +285,11 @@ ui_preparations(struct mknoiseparams *p)
 {
   /* Read the input image as a double type */
   p->input=gal_array_read_one_ch_to_type(p->inputname, p->cp.hdu, NULL,
-                                     GAL_TYPE_FLOAT64, p->cp.minmapsize);
+                                         GAL_TYPE_FLOAT64, p->cp.minmapsize,
+                                         p->cp.quietmmap);
   p->input->wcs=gal_wcs_read(p->inputname, p->cp.hdu, 0, 0, &p->input->nwcs);
+  p->input->ndim=gal_dimension_remove_extra(p->input->ndim, p->input->dsize,
+                                            p->input->wcs);
 
 
   /* If we are dealing with an input table, make sure the format of the
@@ -434,6 +438,7 @@ ui_free_report(struct mknoiseparams *p, struct timeval *t1)
   /* Free the allocated arrays: */
   free(p->cp.hdu);
   free(p->cp.output);
+  gsl_rng_free(p->rng);
   gal_data_free(p->input);
 
   /* Print the final message. */
