@@ -537,6 +537,64 @@ gal_statistics_quantile_function(gal_data_t *input, gal_data_t *value,
 
 
 
+/* Pull out unique elements */
+#define UNIQUE_BYTYPE(TYPE) {                                           \
+    size_t i, j;                                                        \
+    TYPE *a=out->array, b;                                              \
+                                                                        \
+    /* Write the blank value for this type into `b'. */                 \
+    gal_blank_write(&b, out->type);                                     \
+                                                                        \
+    /* Go over the elements, and set the duplicates to blank. */        \
+    /* Note that for integers and floats, the behavior of blank/NaN */  \
+    /* differs: for floats (NaN), we can identify a blank using the  */ \
+    /* fact that by definition, NaN!=NaN. */                            \
+    if(b==b)                                                            \
+      for(i=0;i<out->size;++i)                                          \
+        { if(a[i]!=b)    for(j=i+1;j<out->size;++j) if(a[i]==a[j]) a[j]=b;} \
+    else                                                                \
+      for(i=0;i<out->size;++i)                                          \
+        { if(a[i]==a[i]) for(j=i+1;j<out->size;++j) if(a[i]==a[j]) a[j]=b;} \
+  }
+
+gal_data_t *
+gal_statistics_unique(gal_data_t *input, int inplace)
+{
+  gal_data_t *out = inplace ? input : gal_data_copy(input);
+
+  /* Since we are replacing the repeated elements with blank, re-set the
+     blank flags. */
+  out->flag &= ~GAL_DATA_FLAG_BLANK_CH; /* Set bit to 0. */
+  out->flag &= ~GAL_DATA_FLAG_HASBLANK; /* Set bit to 0. */
+
+  /* Set all non-unique elements to blank. */
+  switch(out->type)
+    {
+    case GAL_TYPE_UINT8:   UNIQUE_BYTYPE( uint8_t  ); break;
+    case GAL_TYPE_INT8:    UNIQUE_BYTYPE( int8_t   ); break;
+    case GAL_TYPE_UINT16:  UNIQUE_BYTYPE( uint16_t ); break;
+    case GAL_TYPE_INT16:   UNIQUE_BYTYPE( int16_t  ); break;
+    case GAL_TYPE_UINT32:  UNIQUE_BYTYPE( uint32_t ); break;
+    case GAL_TYPE_INT32:   UNIQUE_BYTYPE( int32_t  ); break;
+    case GAL_TYPE_UINT64:  UNIQUE_BYTYPE( uint64_t ); break;
+    case GAL_TYPE_INT64:   UNIQUE_BYTYPE( int64_t  ); break;
+    case GAL_TYPE_FLOAT32: UNIQUE_BYTYPE( float    ); break;
+    case GAL_TYPE_FLOAT64: UNIQUE_BYTYPE( double   ); break;
+    default:
+      error(EXIT_FAILURE, 0, "the `unique' operator doesn't support type "
+            "code `%u'", out->type);
+    }
+
+  /* Remove all blank elements (note that `gal_blank_remove' also corrects
+     the size of the dataset and sets it to 1D). */
+  gal_blank_remove_realloc(out);
+  return out;
+}
+
+
+
+
+
 
 
 
@@ -2049,9 +2107,19 @@ gal_statistics_sigma_clip(gal_data_t *input, float multip, float param,
              out of the loop. Normally, `oldstd' should be larger than std,
              because the possible outliers have been removed. If it is not,
              it means that we have clipped too much and must stop anyway,
-             so we don't need an absolute value on the difference! */
-          if( bytolerance && num>0 && ((oldstd - *std) / *std) < param )
-            { gal_data_free(meanstd); gal_data_free(median_d); break; }
+             so we don't need an absolute value on the difference!
+
+             Note that when all the elements are identical after the clip,
+             `std' will be zero. In this case we shouldn't calculate the
+             tolerance (because it will be infinity and thus lager than the
+             requested tolerance level value).*/
+          if( bytolerance && num>0 )
+            if( *std==0 || ((oldstd - *std) / *std) < param )
+              {
+                if(*std==0) {oldmed=*med; oldstd=*std; oldmean=*mean;}
+                gal_data_free(meanstd);   gal_data_free(median_d);
+                break;
+              }
 
           /* Clip all the elements outside of the desired range: since the
              array is sorted, this means to just change the starting
