@@ -670,7 +670,7 @@ gal_wcs_pixel_scale(struct wcsprm *wcs)
   permutation=gal_pointer_allocate(GAL_TYPE_SIZE_T, n, 0, __func__,
                                    "permutation");
   pixscale=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &n, NULL,
-                          0, -1, NULL, NULL, NULL);
+                          0, -1, 1, NULL, NULL, NULL);
 
 
   /* To avoid confusing issues with floating point errors being written in
@@ -847,6 +847,10 @@ wcs_convert_sanity_check_alloc(gal_data_t *coords, struct wcsprm *wcs,
   gal_data_t *tmp;
   size_t ndim=0, firstsize=0, size=coords->size;
 
+  /* Make sure a WCS structure is actually given. */
+  if(wcs==NULL)
+    error(EXIT_FAILURE, 0, "%s: input WCS structure is NULL", func);
+
   for(tmp=coords; tmp!=NULL; tmp=tmp->next)
     {
       /* Count how many coordinates are given. */
@@ -943,7 +947,8 @@ wcs_convert_prepare_out(gal_data_t *coords, struct wcsprm *wcs, int inplace)
     for(i=0;i<wcs->naxis;++i)
       gal_list_data_add_alloc(&out, NULL, GAL_TYPE_FLOAT64, 1,
                               &coords->size, NULL, 0, coords->minmapsize,
-                              wcs->ctype[i], wcs->cunit[i], NULL);
+                              coords->quietmmap, wcs->ctype[i], wcs->cunit[i],
+                              NULL);
   return out;
 }
 
@@ -960,13 +965,13 @@ gal_data_t *
 gal_wcs_world_to_img(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 {
   gal_data_t *out;
-  int status, *stat=NULL, ncoord=coords->size, nelem=wcs->naxis;
+  int status, *stat=NULL, ncoord=coords->size, nelem;
   double *phi=NULL, *theta=NULL, *world=NULL, *pixcrd=NULL, *imgcrd=NULL;
 
   /* Some sanity checks. */
   wcs_convert_sanity_check_alloc(coords, wcs, __func__, &stat, &phi, &theta,
                                  &world, &pixcrd, &imgcrd);
-
+  nelem=wcs->naxis; /* We have to make sure a WCS is given first. */
 
   /* Write the values from the input list of separate columns into a single
      array (WCSLIB input). */
@@ -985,8 +990,9 @@ gal_wcs_world_to_img(gal_data_t *coords, struct wcsprm *wcs, int inplace)
     size_t i;
     printf("\n\n%s sanity check:\n", __func__);
     for(i=0;i<coords->size;++i)
-      printf("(%g, %g) --> (%g, %g), [stat: %d]\n", world[i*2], world[i*2+1],
-             pixcrd[i*2], pixcrd[i*2+1], stat[i]);
+      printf("(%g, %g, %g) --> (%g, %g, %g), [stat: %d]\n",
+              world[i*3],  world[i*3+1 ], world[i*3+2],
+             pixcrd[i*3], pixcrd[i*3+1], pixcrd[i*3+2], stat[i]);
   }
   */
 
@@ -1020,12 +1026,13 @@ gal_data_t *
 gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 {
   gal_data_t *out;
-  int status, *stat=NULL, ncoord=coords->size, nelem=wcs->naxis;
+  int status, *stat=NULL, ncoord=coords->size, nelem;
   double *phi=NULL, *theta=NULL, *world=NULL, *pixcrd=NULL, *imgcrd=NULL;
 
   /* Some sanity checks. */
   wcs_convert_sanity_check_alloc(coords, wcs, __func__, &stat, &phi, &theta,
                                  &world, &pixcrd, &imgcrd);
+  nelem=wcs->naxis; /* We have to make sure a WCS is given first. */
 
 
   /* Write the values from the input list of separate columns into a single
@@ -1045,8 +1052,9 @@ gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
     size_t i;
     printf("\n\n%s sanity check:\n", __func__);
     for(i=0;i<coords->size;++i)
-      printf("(%g, %g) --> (%g, %g), [stat: %d]\n", pixcrd[i*2], pixcrd[i*2+1],
-             world[i*2],  world[i*2+1], stat[i]);
+      printf("(%g, %g, %g) --> (%g, %g, %g), [stat: %d]\n",
+             pixcrd[i*3], pixcrd[i*3+1], pixcrd[i*3+2],
+             world[i*3],  world[i*3+1],  world[i*3+2], stat[i]);
   }
   */
 
